@@ -7,7 +7,11 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2019 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. University of Heidelberg, Heidelberg, Germany
+ *
+ * Copyright (C) 2013-2018 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -97,16 +101,23 @@
 #define SBMLExtension_h
 
 
-#ifndef EXTENSION_CREATE_NS
-#define EXTENSION_CREATE_NS(type,variable,sbmlns)\
+#ifndef EXTENSION_CREATE_NS_WITH_VERSION
+#define EXTENSION_CREATE_NS_WITH_VERSION(type,variable,sbmlns, version)\
   type* variable;\
   {\
       XMLNamespaces* xmlns = sbmlns->getNamespaces();\
       variable = dynamic_cast<type*>(sbmlns);\
       if (variable == NULL)\
       {\
-       variable = new type(sbmlns->getLevel(), sbmlns->getVersion());\
-       for (int i = 0; i < xmlns->getNumNamespaces(); i++)\
+       try\
+ {\
+       variable = new type(sbmlns->getLevel(), sbmlns->getVersion(), version);\
+ }\
+ catch (SBMLExtensionException e)\
+ {\
+        variable = new type(sbmlns->getLevel(), 1, version);\
+}\
+       for (int i = 0; xmlns != NULL && i < xmlns->getNumNamespaces(); ++i)\
        {\
          if (!variable->getNamespaces()->hasURI(xmlns->getURI(i)))\
            variable->getNamespaces()->add(xmlns->getURI(i), xmlns->getPrefix(i));\
@@ -116,6 +127,12 @@
   }
 #endif
 
+#ifndef EXTENSION_CREATE_NS
+#define EXTENSION_CREATE_NS(type,variable,sbmlns)\
+  EXTENSION_CREATE_NS_WITH_VERSION(type,variable,sbmlns,1)
+#endif
+
+
 #include <sbml/common/libsbml-config-common.h>
 
 #include <sbml/extension/SBasePluginCreatorBase.h>
@@ -125,7 +142,7 @@
   /** @cond doxygenLibsbmlInternal */
 #ifndef SWIG
 typedef struct {
-  const char * ref_l3v1;
+  const char * ref_l3v1v1;
 } packageReferenceEntry;
 
 
@@ -133,11 +150,28 @@ typedef struct {
   unsigned int code;
   const char*  shortMessage;
   unsigned int category;
-  unsigned int l3v1_severity;
+  unsigned int l3v1v1_severity;
   const char*  message;
   packageReferenceEntry reference;
 } packageErrorTableEntry;
 
+typedef struct {
+  const char * ref_l3v1v1;
+  const char * ref_l3v1v2;
+} packageReferenceEntryV2;
+
+
+typedef struct {
+  unsigned int code;
+  const char*  shortMessage;
+  unsigned int category;
+  unsigned int l3v1v1_severity;
+  unsigned int l3v1v2_severity;
+  const char*  message;
+  packageReferenceEntryV2 reference;
+} packageErrorTableEntryV2;
+
+
 #endif
   /** @endcond */
 
@@ -167,7 +201,7 @@ public:
    *
    * This creates a copy of an SBMLExtension object.
    *
-   * @param orig The SBMLExtension object to copy.
+   * @param orig the SBMLExtension object to copy.
    */
   SBMLExtension(const SBMLExtension& orig);
 
@@ -181,7 +215,7 @@ public:
   /**
    * Assignment operator for SBMLExtension.
    *
-   * @param rhs The object whose values are used as the basis of the
+   * @param rhs the object whose values are used as the basis of the
    * assignment.
    */
   SBMLExtension& operator=(const SBMLExtension& rhs);
@@ -193,7 +227,7 @@ public:
    *
    * @copydetails doc_sbaseplugincreator_objects
    *
-   * @param sbaseExt the SBasePluginCreatorBase object to add
+   * @param sbaseExt the SBasePluginCreatorBase object to add.
    *
    * @copydetails doc_returns_success_code
    * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -256,7 +290,6 @@ public:
   const SBasePluginCreatorBase*  getSBasePluginCreator(unsigned int n) const;
 
 
-#ifndef LIBSBML_USE_LEGACY_MATH
   /**
    * Adds the given ASTBasePlugin object to this package
    * extension.
@@ -298,7 +331,6 @@ public:
   bool isSetASTBasePlugin() const;
 
 
-#endif /* LIBSBML_USE_LEGACY_MATH */
 
 #endif // SWIG
 
@@ -560,6 +592,11 @@ if (doc->getLevel() == 2)
    */
   virtual bool isInUse(SBMLDocument *doc) const;
 
+  /** @cond doxygenLibsbmlInternal */
+
+  virtual bool hasMutiplePackageVersions() const;
+
+  /** @endcond */
 
   /** @cond doxygenLibsbmlInternal */
   /*
@@ -573,6 +610,9 @@ if (doc->getLevel() == 2)
 
 #ifndef SWIG
   virtual packageErrorTableEntry getErrorTable(unsigned int index) const;
+
+  virtual packageErrorTableEntryV2 getErrorTableV2(unsigned int index) const;
+
 #endif
   /**
    * @ifnot clike @internal @endif@~
@@ -605,20 +645,16 @@ if (doc->getLevel() == 2)
 
 protected:
   /** @cond doxygenLibsbmlInternal */
-
   bool                                 mIsEnabled;
   SupportedPackageURIList              mSupportedPackageURI;
   std::vector<SBasePluginCreatorBase*> mSBasePluginCreators;
 
-#ifndef LIBSBML_USE_LEGACY_MATH
   ASTBasePlugin*                       mASTBasePlugin;
-#endif /* LIBSBML_USE_LEGACY_MATH */
   /** @endcond */
 
 
 private:
   /** @cond doxygenLibsbmlInternal */
-
   friend class SBMLExtensionRegistry;
 
   /** @endcond */
@@ -637,7 +673,7 @@ BEGIN_C_DECLS
 /**
  * Creates a deep copy of the given SBMLExtension_t structure
  *
- * @param ext the SBMLExtension_t structure to be copied
+ * @param ext the SBMLExtension_t structure to be copied.
  *
  * @return a (deep) copy of the given SBMLExtension_t structure.
  *
@@ -650,7 +686,7 @@ SBMLExtension_clone(SBMLExtension_t* ext);
 /**
  * Frees the given SBMLExtension_t structure
  *
- * @param ext the SBMLExtension_t structure to be freed
+ * @param ext the SBMLExtension_t structure to be freed.
  *
  * @copydetails doc_returns_success_code
  * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
@@ -666,7 +702,7 @@ SBMLExtension_free(SBMLExtension_t* ext);
  * Adds the given SBasePluginCreatorBase_t structure to this package
  * extension.
  *
- * @param ext the SBMLExtension_t structure to be freed
+ * @param ext the SBMLExtension_t structure to be freed.
  * @param sbaseExt the SBasePluginCreatorBase_t structure bound to
  * some SBML element and creates a corresponding SBasePlugin_t structure
  * of this package extension.
@@ -686,7 +722,7 @@ SBMLExtension_addSBasePluginCreator(SBMLExtension_t* ext,
  * Returns an SBasePluginCreatorBase_t structure of this package extension
  * bound to the given extension point.
  *
- * @param ext the SBMLExtension_t structure
+ * @param ext the SBMLExtension_t structure.
  * @param extPoint the SBaseExtensionPoint_t to which the returned
  * SBasePluginCreatorBase_t structure bound.
  *
@@ -705,7 +741,7 @@ SBMLExtension_getSBasePluginCreator(SBMLExtension_t* ext,
  * Returns an SBasePluginCreatorBase_t structure of this package extension
  * with the given index.
  *
- * @param ext the SBMLExtension_t structure
+ * @param ext the SBMLExtension_t structure.
  * @param index the index of the returned SBasePluginCreatorBase_t structure for
  * this package extension.
  *
@@ -722,10 +758,10 @@ SBMLExtension_getSBasePluginCreatorByIndex(SBMLExtension_t* ext,
 /**
  * Returns the number of SBasePlugin_t structures stored in the structure.
  *
- * @param ext the SBMLExtension_t structure
+ * @param ext the SBMLExtension_t structure.
  *
  * @return the number of SBasePlugin_t structures stored in the structure,
- * or LIBSBML_INVALID_OBJECT.
+ * or @sbmlconstant{LIBSBML_INVALID_OBJECT, OperationReturnValues_t}.
  *
  * @memberof SBMLExtension_t
  */
@@ -737,10 +773,10 @@ SBMLExtension_getNumOfSBasePlugins(SBMLExtension_t* ext);
  * Returns the number of supported package namespaces (package versions)
  * for this package extension.
  *
- * @param ext the SBMLExtension_t structure
+ * @param ext the SBMLExtension_t structure.
  *
  * @return the number of supported package namespaces (package versions)
- * for this package extension or LIBSBML_INVALID_OBJECT.
+ * for this package extension or @sbmlconstant{LIBSBML_INVALID_OBJECT, OperationReturnValues_t}.
  *
  * @memberof SBMLExtension_t
  */
@@ -749,14 +785,14 @@ int
 SBMLExtension_getNumOfSupportedPackageURI(SBMLExtension_t* ext);
 
 /**
- * Returns a flag indicating, whether the given URI (package version) is
+ * Returns a flag indicating whether the given URI (package version) is
  * supported by this package extension.
  *
- * @param ext the SBMLExtension_t structure
- * @param uri the package uri
+ * @param ext the SBMLExtension_t structure.
+ * @param uri the package uri.
  *
- * @return true (1) if the given URI (package version) is supported by this
- * package extension, otherwise false (0) is returned.
+ * @return @c 1 (true) if the given URI (package version) is supported by this
+ * package extension, otherwise @c 0 (false) is returned.
  *
  * @memberof SBMLExtension_t
  */
@@ -768,10 +804,10 @@ SBMLExtension_isSupported(SBMLExtension_t* ext, const char* uri);
 /**
  * Returns the package URI (package version) for the given index.
  *
- * @param ext the SBMLExtension_t structure
- * @param index the index of the supported package uri to return
+ * @param ext the SBMLExtension_t structure.
+ * @param index the index of the supported package uri to return.
  *
- * @return the package URI (package version) for the given index or NULL.
+ * @return the package URI (package version) for the given index or @c NULL.
  *
  * @memberof SBMLExtension_t
  */
@@ -783,7 +819,7 @@ SBMLExtension_getSupportedPackageURI(SBMLExtension_t* ext, unsigned int index);
 /**
  * Returns the name of the package extension. (e.g. "layout", "multi").
  *
- * @param ext the SBMLExtension_t structure
+ * @param ext the SBMLExtension_t structure.
  *
  * @return the name of the package extension. (e.g. "layout", "multi").
  *
@@ -797,10 +833,10 @@ SBMLExtension_getName(SBMLExtension_t* ext);
  * Returns the uri corresponding to the given SBML level, SBML version,
  * and package version for this extension.
  *
- * @param ext the SBMLExtension_t structure
- * @param sbmlLevel the level of SBML
- * @param sbmlVersion the version of SBML
- * @param pkgVersion the version of package
+ * @param ext the SBMLExtension_t structure.
+ * @param sbmlLevel the level of SBML.
+ * @param sbmlVersion the version of SBML.
+ * @param pkgVersion the version of the package.
  *
  * @return a string of the package URI
  *
@@ -814,8 +850,8 @@ SBMLExtension_getURI(SBMLExtension_t* ext, unsigned int sbmlLevel,
 /**
  * Returns the SBML level associated with the given URI of this package.
  *
- * @param ext the SBMLExtension_t structure
- * @param uri the string of URI that represents a versions of the package
+ * @param ext the SBMLExtension_t structure.
+ * @param uri the string of URI that represents a versions of the package.
  *
  * @return the SBML level associated with the given URI of this package.
  *
@@ -828,8 +864,8 @@ SBMLExtension_getLevel(SBMLExtension_t* ext, const char* uri);
 /**
  * Returns the SBML version associated with the given URI of this package.
  *
- * @param ext the SBMLExtension_t structure
- * @param uri the string of URI that represents a versions of the package
+ * @param ext the SBMLExtension_t structure.
+ * @param uri the string of URI that represents a versions of the package.
  *
  * @return the SBML version associated with the given URI of this package.
  *
@@ -842,8 +878,8 @@ SBMLExtension_getVersion(SBMLExtension_t* ext, const char* uri);
 /**
  * Returns the package version associated with the given URI of this package.
  *
- * @param ext the SBMLExtension_t structure
- * @param uri the string of URI that represents a versions of the package
+ * @param ext the SBMLExtension_t structure.
+ * @param uri the string of URI that represents a versions of the package.
  *
  * @return the package version associated with the given URI of this package.
  *
@@ -857,8 +893,8 @@ SBMLExtension_getPackageVersion(SBMLExtension_t* ext, const char* uri);
  * This method takes a type code of this package and returns a string
  * representing the code.
  *
- * @param ext the SBMLExtension_t structure
- * @param typeCode the typeCode supported by the package
+ * @param ext the SBMLExtension_t structure.
+ * @param typeCode the typeCode supported by the package.
  *
  * @return the string representing the given typecode, or @c NULL in case an
  * invalid extension was provided.
@@ -874,8 +910,8 @@ SBMLExtension_getStringFromTypeCode(SBMLExtension_t* ext, int typeCode);
  * NULL will be returned if the given uri is not defined in the corresponding
  * package.
  *
- * @param ext the SBMLExtension_t structure
- * @param uri the string of URI that represents one of versions of the package
+ * @param ext the SBMLExtension_t structure.
+ * @param uri the string of URI that represents one of versions of the package.
  *
  * @return an SBMLNamespaces_t structure corresponding to the uri. NULL
  *         will be returned if the given uri is not defined in the corresponding
@@ -890,11 +926,11 @@ SBMLExtension_getSBMLExtensionNamespaces(SBMLExtension_t* ext, const char* uri);
 /**
  * Enable/disable this package.
  *
- * @param ext the SBMLExtension_t structure
- * @param isEnabled the value to set : true (1) (enabled) or false (0) (disabled)
+ * @param ext the SBMLExtension_t structure.
+ * @param isEnabled the value to set : @c 1 (true; enabled) or @c 0 (false; disabled).
  *
- * @return true (1) if this function call succeeded, otherwise false (0)is returned.
- * If the extension is invalid, LIBSBML_INVALID_OBJECT will be returned.
+ * @return @c 1 (true) if this function call succeeded, otherwise @c 0 (false) is returned.
+ * If the extension is invalid, @sbmlconstant{LIBSBML_INVALID_OBJECT, OperationReturnValues_t} will be returned.
  *
  * @memberof SBMLExtension_t
  */
@@ -903,12 +939,12 @@ int
 SBMLExtension_setEnabled(SBMLExtension_t* ext, int isEnabled);
 
 /**
- * Check if this package is enabled (true/1) or disabled (false/0).
+ * Check if this package is enabled or disabled.
  *
- * @param ext the SBMLExtension_t structure
+ * @param ext the SBMLExtension_t structure.
  *
- * @return true if the package is enabled, otherwise false is returned.
- * If the extension is invalid, LIBSBML_INVALID_OBJECT will be returned.
+ * @return @c 1 (true) if the package is enabled, otherwise @c 0 (false) is returned.
+ * If the extension is invalid, @sbmlconstant{LIBSBML_INVALID_OBJECT, OperationReturnValues_t} will be returned.
  *
  * @memberof SBMLExtension_t
  */
