@@ -7,7 +7,11 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2017 jointly by the following organizations:
+ * Copyright (C) 2019 jointly by the following organizations:
+ *     1. California Institute of Technology, Pasadena, CA, USA
+ *     2. University of Heidelberg, Heidelberg, Germany
+ *
+ * Copyright (C) 2013-2018 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -236,6 +240,25 @@
  * 'rem' all as @sbmlconstant{AST_FUNCTION,ASTNodeType_t} with the appropriate
  * name set.
  * </ul>
+ *
+ * @class doc_package_math_settings
+ *
+ * @par
+ * This setting affects whether the math added in a given SBML
+ * Level&nbsp;3 package are parsed as those added MathML elements, 
+ * or whether they are added as generic functions or variables
+ * with those names.
+ *
+ * @class doc_package_math_values
+ *
+ * <ul>
+ * <li> @sbmlconstant{L3P_PARSE_PACKAGE_MATH_DIRECTLY,} (value = @c true): parse any 
+ * string defined in a given SBML L3 package as its corresponding ASTType_t.
+ * <li> @sbmlconstant{L3P_PARSE_PACKAGE_MATH_AS_GENERIC,} (value = @c false): 
+ * parse any string that would have been defined in the given SBML L3 package
+ * as @sbmlconstant{AST_FUNCTION,ASTNodeType_t} or @sbmlconstant{AST_NAME,ASTNodeType_t} 
+ * with the appropriate name set.
+ * </ul>
  */
 
 #ifndef L3ParserSettings_h
@@ -245,6 +268,8 @@
 #include <sbml/common/extern.h>
 #include <sbml/common/sbmlfwd.h>
 #include <sbml/util/StringBuffer.h>
+#include <sbml/math/ASTNodeType.h>
+
 
 
 /**
@@ -380,6 +405,23 @@ typedef enum
 #define L3P_PARSE_L3V2_FUNCTIONS_AS_GENERIC  false
 
 
+ /**
+ * Parse the functions added in distrib.
+ *
+ * @see L3ParserSettings::getParseDistribFunctions()
+ * @see L3ParserSettings::setParseDistribFunctions()
+ */
+#define L3P_PARSE_PACKAGE_MATH_DIRECTLY true
+
+ /**
+ * Parse the '%' symbol as an expanded 'piecewise' function (valid in all levels/versions).
+ *
+ * @see L3ParserSettings::getParseDistribFunctions()
+ * @see L3ParserSettings::setParseDistribFunctions()
+ */
+#define L3P_PARSE_PACKAGE_MATH_AS_GENERIC  false
+
+
 typedef enum
 {
     INFIX_SYNTAX_NAMED_SQUARE_BRACKETS
@@ -388,6 +430,7 @@ typedef enum
 } L3ParserGrammarLineType_t;
 
 
+#include <sbml/extension/ASTBasePlugin.h>
 
 #ifdef __cplusplus
 
@@ -399,9 +442,7 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 class Model;
 class L3Parser;
 
-#ifndef LIBSBML_USE_LEGACY_MATH
 class ASTBasePlugin;
-#endif
 
 class LIBSBML_EXTERN L3ParserSettings
 {
@@ -414,10 +455,7 @@ private:
   bool mAvoCsymbol;
   bool mStrCmpIsCaseSensitive;
   bool mModuloL3v2;
-  bool ml3v2Functions;
-#ifndef LIBSBML_USE_LEGACY_MATH
-  std::vector<ASTBasePlugin*> mPlugins;
-#endif
+  std::map<ExtendedMathType_t, bool> mParsePackages;
   /** @endcond */
 
 public:
@@ -506,7 +544,7 @@ public:
    * @sbmlconstant{L3P_COMPARE_BUILTINS_CASE_SENSITIVE,}, symbols are
    * interpreted in a case-sensitive manner.
    *
-   * @param moduloL3v2 ("modulo L3v2") a flag that controls how the
+   * @param modulol3v2 ("modulo l3v2") a flag that controls how the
    * parser will handle the '%' ('modulo') symbol in formulas.  By default, 
    * the parser will convert 'a % b' to a piecewise function that properly
    * calculates the remainder of a with respect to be, but the parser can
@@ -867,8 +905,8 @@ public:
 
 
   /**
-  * Indicates the current behavior set for handling the '%' sumbol in
-  * mathematical formulas.
+  * Indicates the current behavior set for handling whether to
+  * parse the functions added in L3v2 as that MathML or not.
   *
   * @copydetails doc_l3v2_function_settings
   *
@@ -876,30 +914,47 @@ public:
   * values are as follows:
   * @copydetails doc_l3v2_function_values
   *
-  * @see setParseModuloL3v2(@if java boolean@endif)
+  * @see setParsePackageMath(@if java boolean@endif)
   */
   bool getParseL3v2Functions() const;
 
 
   /**
-   * Set up the plugins for this L3ParserSettings, based on the
-   * SBMLNamespaces object.
-   *
-   * When a SBMLNamespaces object is provided, the parser will only interpret
-   * infix syntax understood by the core libSBML @em plus the packages
-   * indicated by the SBMLNamespaces objects provided.  ASTNode objects
-   * returned by the L3Parser will contain those SBMLNamespaces objects, and
-   * will be used to parse certain constructs that may only be understood by
-   * packages (e.g., vectors for the SBML Level&nbsp;3 "arrays" package).
-   * Note that by default, all packages that were compiled with this version
-   * of libSBML are included, so this function is most useful as a way to
-   * turn @em off certain namespaces, such as might be desired if your tool
-   * does not support vectors, for example.
-   *
-   * @param sbmlns a SBMLNamespaces object to be used.  If @c NULL is given
-   * as the value, all plugins will be loaded.
-   */
-  void setPlugins(const SBMLNamespaces * sbmlns);
+  * Sets the behavior for handling functions added in SBML packages
+  *
+  * @copydetails doc_package_math_settings
+  *
+  * This method lets you tell the parser which behavior to use---either
+  * to parse the functions added in a given package as their built-in counterparts,
+  * or as generic functions with that name (to be defined by SBML as
+  * function definitions).  The two possibilities are
+  * represented using the following constants:
+  *
+  * @copydetails doc_package_math_values
+  *
+  * @param l3v2functions a boolean value (one of the constants
+  * @sbmlconstant{L3P_PARSE_L3V2_FUNCTIONS_DIRECTLY,} or
+  * @sbmlconstant{L3P_PARSE_L3V2_FUNCTIONS_AS_GENERIC,})
+  * indicating how to interpret those function names.
+  *
+  * @see getParsePackageMath()
+  */
+  void setParsePackageMath(ExtendedMathType_t package, bool parsepackage);
+
+
+  /**
+  * Indicates the current behavior set for handling whether to
+  * parse the functions added in the given package as that MathML or not.
+  *
+  * @copydetails doc_package_math_settings
+  *
+  * @return A boolean indicating the behavior currently set.  The possible
+  * values are as follows:
+  * @copydetails doc_package_math_values
+  *
+  * @see setParsePackageMath(@if java boolean@endif)
+  */
+  bool getParsePackageMath(ExtendedMathType_t package) const;
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -951,15 +1006,19 @@ private:
    * Should do caseless string comparison.  Return the type of the function,
    * or AST_UNKNOWN if nothing found.
    */
-  int getPackageFunctionFor(const std::string& name) const;
+  ASTNodeType_t getPackageFunctionFor(const std::string& name) const;
 
 
   /**
-   * Delete the plugin objects.
+   * The user input a string of the form "name" with no parentheses, and we want to know if
+   * 'name' is recognized by a package as being a particular function.  We
+   * already know that it is not used in the Model as the id of an element with mathematical meaning.
+   * Should do caseless string comparison.  Return the type of the symbol,
+   * or AST_UNKNOWN if nothing found.
    */
-  void deletePlugins();
+  ASTNodeType_t getPackageSymbolFor(const std::string& name) const;
+
 
-  /** @endcond */
 };
 
 
