@@ -34,6 +34,8 @@
 using namespace std;
 
 
+#ifdef __cplusplus
+
 LIBSBML_CPP_NAMESPACE_BEGIN
 
 
@@ -405,7 +407,7 @@ SpeciesTypeInstance::writeElements (XMLOutputStream& stream) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -420,7 +422,7 @@ SpeciesTypeInstance::accept (SBMLVisitor& v) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -435,7 +437,7 @@ SpeciesTypeInstance::setSBMLDocument (SBMLDocument* d)
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -451,7 +453,7 @@ SpeciesTypeInstance::enablePackageInternal(const std::string& pkgURI,
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -471,7 +473,7 @@ SpeciesTypeInstance::addExpectedAttributes(ExpectedAttributes& attributes)
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -665,7 +667,7 @@ SpeciesTypeInstance::readAttributes(const XMLAttributes& attributes,
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -695,7 +697,7 @@ SpeciesTypeInstance::writeAttributes (XMLOutputStream& stream) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
 /*
@@ -732,7 +734,7 @@ ListOfSpeciesTypeInstances::clone () const
 
 /*
  * Get a SpeciesTypeInstance from the ListOfSpeciesTypeInstances by index.
-*/
+ */
 SpeciesTypeInstance*
 ListOfSpeciesTypeInstances::get(unsigned int n)
 {
@@ -859,7 +861,7 @@ ListOfSpeciesTypeInstances::createObject(XMLInputStream& stream)
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
   /** @cond doxygenLibsbmlInternal */
@@ -887,12 +889,9 @@ ListOfSpeciesTypeInstances::writeXMLNS(XMLOutputStream& stream) const
 }
 
 
-  /** @endcond doxygenLibsbmlInternal */
+  /** @endcond */
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 SpeciesTypeInstance_t *
 SpeciesTypeInstance_create(unsigned int level, unsigned int version,
@@ -902,9 +901,6 @@ SpeciesTypeInstance_create(unsigned int level, unsigned int version,
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 void
 SpeciesTypeInstance_free(SpeciesTypeInstance_t * sti)
@@ -914,9 +910,6 @@ SpeciesTypeInstance_free(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 SpeciesTypeInstance_t *
 SpeciesTypeInstance_clone(SpeciesTypeInstance_t * sti)
@@ -932,9 +925,6 @@ SpeciesTypeInstance_clone(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 char *
 SpeciesTypeInstance_getId(SpeciesTypeInstance_t * sti)
@@ -946,9 +936,6 @@ SpeciesTypeInstance_getId(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 char *
 SpeciesTypeInstance_getName(SpeciesTypeInstance_t * sti)
@@ -960,9 +947,6 @@ SpeciesTypeInstance_getName(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 char *
 SpeciesTypeInstance_getSpeciesType(SpeciesTypeInstance_t * sti)
@@ -974,9 +958,6 @@ SpeciesTypeInstance_getSpeciesType(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 char *
 SpeciesTypeInstance_getCompartmentReference(SpeciesTypeInstance_t * sti)
@@ -988,9 +969,6 @@ SpeciesTypeInstance_getCompartmentReference(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_isSetId(SpeciesTypeInstance_t * sti)
@@ -999,9 +977,6 @@ SpeciesTypeInstance_isSetId(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_isSetName(SpeciesTypeInstance_t * sti)
@@ -1010,9 +985,6 @@ SpeciesTypeInstance_isSetName(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_isSetSpeciesType(SpeciesTypeInstance_t * sti)
@@ -1021,9 +993,6 @@ SpeciesTypeInstance_isSetSpeciesType(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_isSetCompartmentReference(SpeciesTypeInstance_t * sti)
@@ -1032,9 +1001,6 @@ SpeciesTypeInstance_isSetCompartmentReference(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_setId(SpeciesTypeInstance_t * sti, const char * id)
@@ -1043,9 +1009,6 @@ SpeciesTypeInstance_setId(SpeciesTypeInstance_t * sti, const char * id)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_setName(SpeciesTypeInstance_t * sti, const char * name)
@@ -1054,9 +1017,6 @@ SpeciesTypeInstance_setName(SpeciesTypeInstance_t * sti, const char * name)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_setSpeciesType(SpeciesTypeInstance_t * sti, const char * speciesType)
@@ -1065,9 +1025,6 @@ SpeciesTypeInstance_setSpeciesType(SpeciesTypeInstance_t * sti, const char * spe
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_setCompartmentReference(SpeciesTypeInstance_t * sti, const char * compartmentReference)
@@ -1076,9 +1033,6 @@ SpeciesTypeInstance_setCompartmentReference(SpeciesTypeInstance_t * sti, const c
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_unsetId(SpeciesTypeInstance_t * sti)
@@ -1087,9 +1041,6 @@ SpeciesTypeInstance_unsetId(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_unsetName(SpeciesTypeInstance_t * sti)
@@ -1098,9 +1049,6 @@ SpeciesTypeInstance_unsetName(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_unsetSpeciesType(SpeciesTypeInstance_t * sti)
@@ -1109,9 +1057,6 @@ SpeciesTypeInstance_unsetSpeciesType(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_unsetCompartmentReference(SpeciesTypeInstance_t * sti)
@@ -1120,9 +1065,6 @@ SpeciesTypeInstance_unsetCompartmentReference(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 int
 SpeciesTypeInstance_hasRequiredAttributes(SpeciesTypeInstance_t * sti)
@@ -1131,9 +1073,6 @@ SpeciesTypeInstance_hasRequiredAttributes(SpeciesTypeInstance_t * sti)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 SpeciesTypeInstance_t *
 ListOfSpeciesTypeInstances_getById(ListOf_t * lo, const char * sid)
@@ -1145,9 +1084,6 @@ ListOfSpeciesTypeInstances_getById(ListOf_t * lo, const char * sid)
 }
 
 
-/**
- * 
- */
 LIBSBML_EXTERN
 SpeciesTypeInstance_t *
 ListOfSpeciesTypeInstances_removeById(ListOf_t * lo, const char * sid)
@@ -1163,4 +1099,6 @@ ListOfSpeciesTypeInstances_removeById(ListOf_t * lo, const char * sid)
 
 LIBSBML_CPP_NAMESPACE_END
 
+#endif /*__cplusplus */
+
 
